#include "leqsolve.h"

/* =====================================  Utility function */
static void
prodmv(int N, const double I[], double d, const double V[], double X[]) {
    int i, j, k=0;
    for (i=0; i<N; ++i) {
        double sum = I[k] * V[0];
	for (j=1; j<N; ++j) sum += I[k+j] * V[j];
	k +=N;
	X[i] = sum/d;
    }
}

/* =====================================  Solve a set of 2 linear equations. */
int 
solve22(const double A[4], const double V[2], double X[2]) {
    double det = A[0]*A[3] - A[1]*A[2];
    if (det == 0) return 1;
    double Inv[4];
    Inv[0] =  A[3];
    Inv[1] = -A[2];
    Inv[2] = -A[1];
    Inv[3] =  A[0];
    X[0] = (Inv[0] * V[0] + Inv[1] * V[1])/det;
    X[1] = (Inv[2] * V[0] + Inv[3] * V[1])/det;
    return 0;
}

/* =====================================  Solve a set of 3 linear equations. */
int 
solve33(const double A[9], const double V[3], double X[3]) {
    double Inv[9];
    Inv[0] =  (A[4]*A[8] - A[5]*A[7]);
    Inv[1] = -(A[3]*A[8] - A[6]*A[5]);
    Inv[2] =  (A[3]*A[7] - A[4]*A[6]);

    double det = A[0]*Inv[0] + A[1]*Inv[1] + A[2]*Inv[2];
    if (det == 0) return 1;

    Inv[3] = -(A[1]*A[8] - A[2]*A[7]);
    Inv[4] =  (A[0]*A[8] - A[2]*A[6]);
    Inv[5] = -(A[0]*A[7] - A[6]*A[1]);

    Inv[6] =  (A[1]*A[5] - A[4]*A[2]);
    Inv[7] = -(A[0]*A[5] - A[2]*A[3]);
    Inv[8] =  (A[0]*A[4] - A[1]*A[3]);

    prodmv(3, Inv, det, V, X);
    return 0;
}
